@@ -295,13 +295,13 @@ sa_rc_t sa_s2a(sa_addr_t **saa, const struct sockaddr *sabuf, socklen_t salen)
     /* fill in family */
     sf = 0;
     if (sizeof(struct sockaddr_in) == salen) {
-        sa4 = (struct sockaddr_in *)sabuf;
+        sa4 = (struct sockaddr_in *)((void *)sabuf);
         if (sa4->sin_family == AF_INET)
             sf = AF_INET;
     }
 #ifdef AF_INET6
     else if (sizeof(struct sockaddr_in6) == salen) {
-        sa6 = (struct sockaddr_in6 *)sabuf;
+        sa6 = (struct sockaddr_in6 *)((void *)sabuf);
         if (sa6->sin6_family == AF_INET6)
             sf = AF_INET6;
     }
@@ -332,13 +332,13 @@ sa_rc_t sa_a2u(const sa_addr_t *saa, char **uri)
     if ((pe = getprotobynumber(saa->saa_proto)) == NULL)
         return SA_ERR_SYS;
     if (saa->saa_family == AF_INET) {
-        sa4 = (struct sockaddr_in *)saa->saa_buf;
+        sa4 = (struct sockaddr_in *)((void *)saa->saa_buf);
         inet_ntop(AF_INET, &sa4->sin_addr.s_addr, caHost, sizeof(caHost));
         nPort = ntohs(sa4->sin_port);
     }
 #ifdef AF_INET6
     else if (saa->saa_family == AF_INET6) {
-        sa6 = (struct sockaddr_in6 *)saa->saa_buf;
+        sa6 = (struct sockaddr_in6 *)((void *)saa->saa_buf);
         inet_ntop(AF_INET6, &sa6->sin6_addr.s6_addr, caHost, sizeof(caHost));
         nPort = ntohs(sa6->sin6_port);
     }
