@@ -1,31 +1,28 @@
 /*
- *  lmtp.c: LMTP library (implementation)
- *
- *  Copyright (c) 2001 The OSSP Project (http://www.ossp.org/)
- *  Copyright (c) 2001 Cable & Wireless Deutschland (http://www.cw.com/de/)
- *
- *  This file is part of OSSP lmtp2nntp, an LMTP speaking local
- *  mailer which forwards mails as Usenet news articles via NNTP.
- *  It can be found at http://www.ossp.com/pkg/lmtp2nntp/.
- *
- *  Permission to use, copy, modify, and distribute this software for
- *  any purpose with or without fee is hereby granted, provided that
- *  the above copyright notice and this permission notice appear in all
- *  copies.
- *
- *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- *  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
- *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- *  SUCH DAMAGE.
- */
+**  Copyright (c) 2001 The OSSP Project <http://www.ossp.org/>
+**  Copyright (c) 2001 Cable & Wireless Deutschland <http://www.cw.com/de/>
+**
+**  This file is part of OSSP lmtp2nntp, an LMTP speaking local
+**  mailer which forwards mails as Usenet news articles via NNTP.
+**  It can be found at http://www.ossp.org/pkg/lmtp2nntp/.
+**
+**  This program is free software; you can redistribute it and/or
+**  modify it under the terms of the GNU General Public  License
+**  as published by the Free Software Foundation; either version
+**  2.0 of the License, or (at your option) any later version.
+**
+**  This program is distributed in the hope that it will be useful,
+**  but WITHOUT ANY WARRANTY; without even the implied warranty of
+**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+**  General Public License for more details.
+**
+**  You should have received a copy of the GNU General Public License
+**  along with this file; if not, write to the Free Software
+**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+**  USA, or contact the OSSP project <ossp@ossp.org>.
+**
+**  lmtp.c: Local Mail Transfer Protocol (LMTP) server library
+*/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,6 +70,21 @@ struct lmtp_st {
     int               wfd;      /* file descriptor for writing */
 };
 
+static int lmtp_select(void *ctx, int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds, struct timeval *tv)
+{
+    return select(nfds, rfds, wfds, efds, tv);
+}
+
+static ssize_t lmtp_read(void *ctx, int fd, void *buf, size_t buflen)
+{
+    return read(fd, buf, buflen);
+}
+
+static ssize_t lmtp_write(void *ctx, int fd, const void *buf, size_t buflen)
+{
+    return write(fd, buf, buflen);
+}
+
 static int verbindex(lmtp_t *lmtp, char *verb)
 {
     /* returns the index of the verb or -1 if verb not registered */
@@ -113,13 +125,15 @@ lmtp_t *lmtp_create(int rfd, int wfd, lmtp_io_t *io)
         return NULL;
 
     if (io == NULL) {
-        lmtp->io.select = select;
-        lmtp->io.read   = read;
-        lmtp->io.write  = write;
+        lmtp->io.ctx    = NULL;
+        lmtp->io.select = lmtp_select;
+        lmtp->io.read   = lmtp_read;
+        lmtp->io.write  = lmtp_write;
     } else {
-        lmtp->io.select = io->select ? io->select : select;
-        lmtp->io.read   = io->read   ? io->read   : read;
-        lmtp->io.write  = io->write  ? io->write  : write;
+        lmtp->io.ctx    = io->ctx;
+        lmtp->io.select = io->select ? io->select : lmtp_select;
+        lmtp->io.read   = io->read   ? io->read   : lmtp_read;
+        lmtp->io.write  = io->write  ? io->write  : lmtp_write;
     }
 
     lmtp->rl.rl_cnt = 0;
@@ -167,7 +181,7 @@ lmtp_rc_t lmtp_readline(lmtp_t *lmtp, char *buf, size_t buflen)
         /* fetch one character (but read more) */
         if (rl->rl_cnt <= 0) {
             do {
-                rl->rl_cnt = lmtp->io.read(lmtp->rfd, rl->rl_buf, LMTP_LINE_MAXLEN);
+                rl->rl_cnt = lmtp->io.read(lmtp->io.ctx, lmtp->rfd, rl->rl_buf, LMTP_LINE_MAXLEN);
             } while (rl->rl_cnt == -1 && errno == EINTR);
             if (rl->rl_cnt == -1)
                 return LMTP_ERR_SYSTEM;
@@ -356,7 +370,7 @@ lmtp_rc_t lmtp_response(lmtp_t *lmtp, lmtp_res_t *res)
         formatbuf[len++] = '\r';
         formatbuf[len++] = '\n';
         do {
-            rv = lmtp->io.write(lmtp->wfd, formatbuf, len);
+            rv = lmtp->io.write(lmtp->io.ctx, lmtp->wfd, formatbuf, len);
         } while (rv == -1 && errno == EINTR);
         if (rv == -1)
             return LMTP_ERR_SYSTEM;
