@@ -43,8 +43,8 @@
 /* library version check (compile-time) */
 #define  L2_VERSION_HEX_REQ 0x001200
 #define  L2_VERSION_STR_REQ "0.1.0"
-#define STR_VERSION_HEX_REQ 0x009205
-#define STR_VERSION_STR_REQ "0.9.5"
+#define STR_VERSION_HEX_REQ 0x009206
+#define STR_VERSION_STR_REQ "0.9.6"
 #ifdef L2_VERSION_HEX
 #if L2_VERSION_HEX < L2_VERSION_HEX_REQ
 #error "require a newer version of OSSP L2"
@@ -83,7 +83,7 @@
 #define NUL '\0'
 #endif
 
-#define ERR_EXECUTION -1
+#define ERR_EXECUTION 1
 #define ERR_DELIVERY -2
 
 #define STDSTRLEN 128
@@ -121,6 +121,7 @@ struct ns {
 };
 
 typedef struct {
+    l2_context_t    ctx;
     char           *progname;
     char           *option_logfile;
     int             option_groupmode;
@@ -316,6 +317,7 @@ int main(int argc, char **argv)
     /* create application context */
     if ((ctx = (lmtp2nntp_t *)malloc(sizeof(lmtp2nntp_t))) == NULL)
         CU(ERR_EXECUTION);
+    ctx->ctx.vp = ctx;
     ctx->progname = strdup(argv[0]);
     ctx->option_logfile = NULL;
     ctx->option_groupmode = GROUPMODE_ARG;
@@ -555,7 +557,7 @@ int main(int argc, char **argv)
         CU(ERR_EXECUTION);
     }
 
-    if (l2_stream_formatter(ctx->l2, 'P', formatter_prefix, (l2_context_t *)&ctx) != L2_OK) {
+    if (l2_stream_formatter(ctx->l2, 'P', formatter_prefix, &ctx->ctx) != L2_OK) {
         fprintf(stderr, "%s:Error: logging failed to register formatter\n", ctx->progname);
         CU(ERR_EXECUTION);
     }
@@ -634,7 +636,6 @@ int main(int argc, char **argv)
     signal(SIGSEGV, (void(*)())catchsignal);
     signal(SIGSYS,  (void(*)())catchsignal);
     signal(SIGTERM, (void(*)())catchsignal);
-    signal(SIGINFO,            SIG_IGN    );
     signal(SIGUSR1,            SIG_IGN    );
     signal(SIGUSR2,            SIG_IGN    );
 
